﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Chapter1
{
    class Item08
    {
        public static void Tests()
        {
            var list = Enumerable.Range(1, 50);

            // assign the result to a tuple variable:
            var reuslt = FindFirstOccurence(list, 42);
            Console.WriteLine($"The first occurence of {reuslt.sought} is {reuslt.index}");
            // assign the result to different variables:
            (int number, int index) = FindFirstOccurence(list, 42);
            Console.WriteLine($"The first occurence of {number} is {index}");
        }


        // Anonymous Types section:

        private static void MethodSample()
        {
            IDictionary<int, string> numberDescriptionDictionary =
                new Dictionary<int, string>()
            {
                {1,"one"},
                {2, "two"},
                {3, "three"},
                {4, "four"},
                {5, "five"},
                {6, "six"},
                {7, "seven"},
                {8, "eight"},
                {9, "nine"},
                {10, "ten"},
            };
            List<int> numbers = new List<int>()
                { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10 };
            var r = from n in numbers
                    where n % 2 == 0
                    select new
                    {
                        Number = n,
                        Description = numberDescriptionDictionary[n]
                    };
            r = from n in FindValue(r, 
                new { Number = 2, Description = "two" })
                select n;
        }

        static (T sought, int index) FindFirstOccurence<T>(
            IEnumerable<T> enumerable, T value)
        {
            int index = 0;
            foreach (T element in enumerable)
            {
                if (element.Equals(value))
                {
                    return (value, index);
                }
                index++;
            }
            return (default(T), -1);
        }

        static IEnumerable<T> FindValue<T>(IEnumerable<T> enumerable,
            T value)
        {
            foreach (T element in enumerable)
            {
                if (element.Equals(value))
                {
                    yield return element;
                }
            }
        }
        private static void sampletwo()
        {
            Random randomNumbers = new Random();
            var sequence = (from x in Utilities.Generator(100,
                                () => randomNumbers.NextDouble() * 100)
                            let y = randomNumbers.NextDouble() * 100
                            select new { x, y }).TakeWhile(
                           point => point.x < 75);
        }

        private static void SampleThree()
        {
            Random randomNumbers = new Random();

            // Usage:
            var sequence = (from x in Utilities.Generator(100,
                                () => randomNumbers.NextDouble() * 100)
                            let y = randomNumbers.NextDouble() * 100
                            select new { x, y }).TakeWhile(
                           point => point.x < 75);

            var scaled = sequence.Map(p => 
            new {
                x = p.x * 5,
                y = p.y * 5 }
            );
            var translated = scaled.Map(p => 
            new {
                x = p.x - 20,
                y = p.y - 20
            });
            var distances = translated.Map(p => new
            {
                p.x,
                p.y,
                distance = Math.Sqrt(p.x * p.x + p.y * p.y)
            });
            var filtered = from location in distances
                           where location.distance < 500.0
                           select location;

        }

    }

    public static class Utilities
    {
        public static IEnumerable<double> Generator(int num,
            Func<double> generatorFunc)
        {
            for (var i = 0; i < num; i++)
                yield return generatorFunc();
        }

        // In another class define the Map function:
        public static IEnumerable<TResult> Map<TSource, TResult>
            (this IEnumerable<TSource> source,
            Func<TSource, TResult> mapFunc)
        {
            foreach (TSource s in source)
                yield return mapFunc(s);
        }


    }
}


