﻿using System;
using static System.Console;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Chapter2
{
    class IncludedFromBCL
    {
#if SomeSymbol
        public static bool ReferenceEquals
            (object left, object right);
public static bool Equals(object left, object right)
{
    // Check object identity
    if (Object.ReferenceEquals(left, right) )
        return true;
    // both null references handled above
    if (Object.ReferenceEquals(left, null) ||
        Object.ReferenceEquals(right, null))
        return false;
    return left.Equals(right);
}
        public virtual bool Equals(object right);
        public static bool operator ==(MyClass left, MyClass right);
#endif

    }
    class Item09
    {
        private static void Snippet01()
        {
            int i = 5;
            int j = 5;
            if (Object.ReferenceEquals(i, j))
                WriteLine("Never happens.");
            else
                WriteLine("Always happens.");

            if (Object.ReferenceEquals(i, i))
                WriteLine("Never happens.");
            else
                WriteLine("Always happens.");

        }

        public static void Tests()
        {
            //Test:
            B baseObject = new B();
            D derivedObject = new D();

            // Comparison 1.
            if (baseObject.Equals(derivedObject))
                WriteLine("Equals");
            else
                WriteLine("Not Equal");

            // Comparison 2.
            if (derivedObject.Equals(baseObject))
                WriteLine("Equals");
            else
                WriteLine("Not Equal");
        }

    }

    public class Foo : IEquatable<Foo>
    {
        public override bool Equals(object right)
        {
            // check null:
            // this pointer is never null in C# methods.
            if (object.ReferenceEquals(right, null))
                return false;

            if (object.ReferenceEquals(this, right))
                return true;

            // Discussed below.
            if (this.GetType() != right.GetType())
                return false;

            // Compare this type's contents here:
            return this.Equals(right as Foo);
        }

        // IEquatable<Foo> Members
        public bool Equals(Foo other)
        {
            // elided.
            return true;
        }
    }

    public class B : IEquatable<B>
    {
        public override bool Equals(object right)
        {
            // check null:
            if (object.ReferenceEquals(right, null))
                return false;

            // Check reference equality:
            if (object.ReferenceEquals(this, right))
                return true;

            // Problems here, discussed below.
            B rightAsB = right as B;
            if (rightAsB == null)
                return false;

            return this.Equals(rightAsB);
        }

        // IEquatable<B> Members

        public bool Equals(B other)
        {
            // elided
            return true; // or false, based on test
        }
    }

    public class D : B, IEquatable<D>
    {
        // etc.
        public override bool Equals(object right)
        {
            // check null:
            if (object.ReferenceEquals(right, null))
                return false;

            if (object.ReferenceEquals(this, right))
                return true;

            // Problems here.
            D rightAsD = right as D;
            if (rightAsD == null)
                return false;

            if (base.Equals(rightAsD) == false)
                return false;

            return this.Equals(rightAsD);
        }

        // IEquatable<D> Members
        public bool Equals(D other)
        {
            // elided.
            return true; // or false, based on test
        }
    }


}
