﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace Chapter2
{
    public abstract class WorkerEngineBase
    {
        public virtual event 
            EventHandler<WorkerEventArgs> OnProgress;

        public void DoLotsOfStuff()
        {
            for (int i = 0; i < 100; i++)
            {
                SomeWork();
                WorkerEventArgs args = new WorkerEventArgs();
                args.Percent = i;
                OnProgress?.Invoke(this, args);
                if (args.Cancel)
                    return;
            }
        }

        protected abstract void SomeWork();
    }

    public class WorkerEngineDerived : WorkerEngineBase
    {
        protected override void SomeWork()
        {
            // elided
        }
    }

    public class Item21
    {
        public static void Tests()
        {

        }
    }

    namespace Step3
    {
        public class WorkerEngineDerived : WorkerEngineBase
        {
            protected override void SomeWork()
            {
                Thread.Sleep(50);
            }
            // Broken. This hides the private event field in
            // the base class
            public override event
                EventHandler<WorkerEventArgs> OnProgress;
        }
    }

    namespace Step4
    {
        public class WorkerEngineDerived : WorkerEngineBase
        {
            protected override void SomeWork()
            {
                Thread.Sleep(50);
            }
            public override event
                EventHandler<WorkerEventArgs> OnProgress
            {
                add
                {
                    base.OnProgress += value;
                }
                remove
                {
                    base.OnProgress -= value;
                }
            }
            // Important: Only the base class can raise the event.
            // Derived cannot raise the events directly.
            // If derived classes should raise events, the base
            // class must provide a protected method to
            // raise the events.
        }
    }

    namespace Step5
    {
        public abstract class WorkerEngineBase
        {
            protected EventHandler<WorkerEventArgs> progressEvent;

            public virtual event
                EventHandler<WorkerEventArgs> OnProgress
            {
                [MethodImpl(MethodImplOptions.Synchronized)]
                add
                {
                    progressEvent += value;
                }
                [MethodImpl(MethodImplOptions.Synchronized)]
                remove
                {
                    progressEvent -= value;
                }
            }

            public void DoLotsOfStuff()
            {
                for (int i = 0; i < 100; i++)
                {
                    SomeWork();
                    WorkerEventArgs args = new WorkerEventArgs();
                    args.Percent = i;
                    progressEvent?.Invoke(this, args);

                    if (args.Cancel)
                        return;
                }
            }

            protected abstract void SomeWork();
        }
        public class WorkerEngineDerived : WorkerEngineBase
        {
            protected override void SomeWork()
            {
                //elided
            }
            // Works. Access base class event field.
            public override event
                EventHandler<WorkerEventArgs> OnProgress
            {
                [MethodImpl(MethodImplOptions.Synchronized)]
                add
                {
                    progressEvent += value;
                }
                [MethodImpl(MethodImplOptions.Synchronized)]
                remove
                {
                    progressEvent -= value;
                }
            }
        }

        public class Container
        {
            public class WorkerEngineDerived : WorkerEngineBase
            {
                protected override void SomeWork()
                {
                    //elided
                }
                // Broken. Private field hides the base class
                public override event
                    EventHandler<WorkerEventArgs> OnProgress;
            }
        }
    }

    namespace Step6
    {
        public abstract class WorkerEngineBase
        {
            public virtual event
                EventHandler<WorkerEventArgs> OnProgress;

            protected virtual WorkerEventArgs
                RaiseEvent(WorkerEventArgs args)
            {
                OnProgress?.Invoke(this, args);
                return args;
            }

            public void DoLotsOfStuff()
            {
                for (int i = 0; i < 100; i++)
                {
                    SomeWork();
                    WorkerEventArgs args = new WorkerEventArgs();
                    args.Percent = i;
                    RaiseEvent(args);
                    if (args.Cancel)
                        return;
                }
            }

            protected abstract void SomeWork();
        }

        public class WorkerEngineDerived : WorkerEngineBase
        {
            protected override void SomeWork()
            {
                Thread.Sleep(50);
            }

            public override event
                EventHandler<WorkerEventArgs> OnProgress;

            protected override WorkerEventArgs
                RaiseEvent(WorkerEventArgs args)
            {
                OnProgress?.Invoke(this, args);
                return args;
            }
        }
    }
}
