﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Chapter3_Task
{
    class Item26
    {
        public object CurrentUser { get; private set; }

        public static void Tests()
        {
            AppDomain.CurrentDomain.UnhandledException += (sender, e) =>
            {
                Console.ForegroundColor = ConsoleColor.Cyan;
                Console.WriteLine(e.ExceptionObject.ToString());
            };

            var startTask = DoAsyncThings();
            FireAndForget();
            startTask.Wait();
            startTask = DoAsyncThings();
            startTask.Wait();
        }

        private static async void FireAndForget()
        {
            var task = DoAsyncThings();
            await task;
            var task2 = ContinueWork();
            await task2;
        }

        private static Task ContinueWork()
        {
            var tcs = new TaskCompletionSource<bool>();
            tcs.TrySetException(new InvalidOperationException());
            return tcs.Task;
        }

        private static async Task DoAsyncThings()
        {
            await Task.Delay(2000);
        }

        public async void SetSessionState()
        {
            var config = await ReadConfigFromNetwork();
            this.CurrentUser = config.User;
        }

        private Task<Person> ReadConfigFromNetwork()
        {
            throw new NotImplementedException();
        }
        
        private void ExampleTestCode()
        {
            /*
            var t = new SessionManager();
            t.SetSessionState();
            // wait a while
            await Task.Delay(1000);
            Assert.Equal(t.User, "TestLibrary User");
            */
        }

        /*
        private async void OnTestCommand(object sender, RoutedEventArgs e)
        {
            var viewModel = (DataContext as SampleViewModel);
            try
            {
                await viewModel.Update();
            }
            catch (Exception ex)
            {
                viewModel.Messages.Add(ex.ToString());
            }
        }

        private async void OnCommand(object sender, RoutedEventArgs e)
        {
            var viewModel = (DataContext as SampleViewModel);
            try
            {
                await viewModel.Update();
            }
            catch (Exception ex) when (logMessage(viewModel, ex))
            {
            }
        }
        private bool logMessage(SampleViewModel viewModel, Exception ex)
        {
            viewModel.Messages.Add(ex.ToString());
            return false;
        }
        */

    }

    public static class Utilities
    {
        public static async void FireAndForget(this Task task, Action<Exception> onErrors)
        {
            try
            {
                await task;
            }
            catch (Exception ex)
            {
                onErrors(ex);
            }
        }

        public static async void FireAndForget(this Task task, Func<Exception, bool> onError)
        {
            try
            {
                await task;
            }
            catch (Exception ex) when (onError(ex))
            {
            }
        }

        public static async void FireAndForget<TException>(this Task task, 
            Action<TException> recovery, Func<Exception, bool> onError)
            where TException : Exception
        {
            try
            {
                await task;
            }
            // relies on onError() logging method 
            // always returning false:
            catch (Exception ex) when (onError(ex))
            {
            }
            catch (TException ex2)
            {
                recovery(ex2);
            }
        }
    }

}
