﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Chapter3_Task
{
    class Item27
    {
        public static void Tests()
        {
            SyncOverAsyncDeadlock();
        }

        public static async Task<int> ComputeUsageAsync()
        {
            try
            {
                var operand = await GetLeftOperandForIndex(19);
                var operand2 = await GetRightOperandForIndex(23);
                return operand + operand2;
            }
            catch (KeyNotFoundException e)
            {
                return 0;
            }
        }

        public static int ComputeUsage()
        {
            try
            {
                var operand = GetLeftOperandForIndex(19).Result;
                var operand2 = GetRightOperandForIndex(23).Result;
                return operand + operand2;
            }
            catch (AggregateException e) 
            when (e.InnerExceptions.FirstOrDefault().GetType() 
            == typeof(KeyNotFoundException))
            {
                return 0;
            }
        }

   
        private static async Task SimulatedWorkAsync()
        {
            await Task.Delay(1000);
        }

        // This method can cause a deadlock in ASP.NET or GUI context.
        public static void SyncOverAsyncDeadlock()
        {
            // Start the task.
            var delayTask = SimulatedWorkAsync();
            // Wait Synchronously for the delay to complete.
            delayTask.Wait();
        }

        public double ComputeValue()
        {
            // Do lots of work.
            double finalAnswer = 0;
            for (int i = 0; i < 10_000_000; i++)
                finalAnswer += InterimCalculation(i);
            return finalAnswer;
        }

        public Task<double> ComputeValueAsync()
        {
            return Task.Run(() => ComputeValue());
        }

        private static Task<int> GetRightOperandForIndex(int left)
        {
            return Task.FromResult(left);
        }

        private static Task<int> GetLeftOperandForIndex(int right)
        {
            return Task.FromResult(right);
        }

        private double InterimCalculation(int i)
        {
            return i * new Random().NextDouble();
        }

    }
}
