﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace Chapter3
{
    class Item27
    {
        public static class Hero
        {
            public static double FindRoot(double number)
            {
                double previousError = double.MaxValue;
                double guess = 1;
                double error = Math.Abs(guess * guess - number);

                while (previousError / error > 1.000001)
                {
                    guess = (number / guess + guess) / 2.0;
                    previousError = error;
                    error = Math.Abs(guess * guess - number);
                }
                return guess;
            }
        }

        const int LowerBound = 2;
        static int UpperBound = 10000;

        public static void Tests()
        {
            UpperBound = 1_000_000;

            var tests = new int[] {2, 18, 98, 498, 2498, 12498, 62498, 312498};
             
            Console.WriteLine("Number of threads, One Thread, Tasks, ThreadPool, ManualThreads");
            foreach (int threads in tests)
            {
                double single = OneThread();
                var task = TaskLibrary(threads).Result;
                double threadPool = ThreadPoolThreads(threads);
                double manual = ManualThreads(threads);
                Console.WriteLine($"{threads}, {single}, {task}, {threadPool}, {manual}");
            }
        }

        private static double OneThread()
        {
            Stopwatch start = new Stopwatch();
            double answer;
            start.Start();
            for (int i = LowerBound; i < UpperBound; i++)
                answer = Hero.FindRoot(i);
            start.Stop();
            return start.ElapsedMilliseconds;
        }

        private static async Task<double> TaskLibrary(int numTasks)
        {
            var itemsPerTask = (UpperBound - LowerBound) / numTasks + 1;
            double answer;
            List<Task> tasks = new List<Task>(numTasks);
            Stopwatch start = new Stopwatch();
            start.Start();
            for(int i = LowerBound; i < UpperBound; i+= itemsPerTask)
            {
                tasks.Add(Task.Run(() =>
                {
                    for (int j = i; j < i + itemsPerTask; j++)
                        answer = Hero.FindRoot(j);
                }));
            }
            await Task.WhenAll(tasks);
            start.Stop();
            return start.ElapsedMilliseconds;
        }

        private static double ThreadPoolThreads(int numThreads)
        {
            Stopwatch start = new Stopwatch();
            using (AutoResetEvent e = new AutoResetEvent(false))
            {
                int workerThreads = numThreads;
                double answer;
                start.Start();
                for (int thread = 0; thread < numThreads; thread++)
                    System.Threading.ThreadPool.QueueUserWorkItem(
                        (x) =>
                        {
                            for (int i = LowerBound;
                                i < UpperBound; i++)
                                if (i % numThreads == thread)
                                    answer = Hero.FindRoot(i);
                            if (Interlocked.Decrement(ref workerThreads) == 0)
                                e.Set();
                        });
                e.WaitOne();
                start.Stop();
                return start.ElapsedMilliseconds;
            }
        }

        private static double ManualThreads(int numThreads)
        {
            Stopwatch start = new Stopwatch();
            using (AutoResetEvent e = new AutoResetEvent(false))
            {
                int workerThreads = numThreads;
                double answer;
                start.Start();
                for (int thread = 0; thread < numThreads; thread++)
                {
                    System.Threading.Thread t = new Thread(
                        () =>
                        {
                            for (int i = LowerBound;
                                i < UpperBound; i++)
                                if (i % numThreads == thread)
                                    answer = Hero.FindRoot(i);
                            if (Interlocked.Decrement(ref workerThreads) == 0)
                                e.Set();
                        });
                    t.Start();
                }
                e.WaitOne();
                start.Stop();
                return start.ElapsedMilliseconds;
            }
        }

    }
}
