﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace ThreadsControlInvoke
{
    public static class ControlExtensions
    {
        public static void InvokeIfNeeded(this Control ctl, Action doit)
        {
            if (ctl.IsHandleCreated == false)
                doit();
            else if (ctl.InvokeRequired)
                ctl.Invoke(doit);
            else
                doit();
        }

        public static void InvokeIfNeeded<T>(this Control ctl,
            Action<T> doit, T args)
        {
            if (ctl.IsHandleCreated == false)
                throw new InvalidOperationException("Window handle for ctl has not been created");
            else if (ctl.InvokeRequired)
                ctl.Invoke(doit, args);
            else
                doit(args);
        }

        // versions built on 3,4 parameters elided

        public static void InvokeAsync(this Control ctl, Action doit)
        {
            ctl.BeginInvoke(doit);
        }

        public static void InvokeAsync<T>(this Control ctl,
            Action<T> doit, T args)
        {
            ctl.BeginInvoke(doit, args);
        }
    }

    public static class WPFControlExtensions
    {
        public static void InvokeIfNeeded(
            this System.Windows.Threading.DispatcherObject ctl,
            Action doit,
            System.Windows.Threading.DispatcherPriority priority)
        {
            if (System.Threading.Thread.CurrentThread !=
                ctl.Dispatcher.Thread)
            {
                ctl.Dispatcher.Invoke(priority,
                    doit);
            }
            else
            {
                doit();
            }
        }
        public static void InvokeIfNeeded<T>(
            this System.Windows.Threading.DispatcherObject ctl,
            Action<T> doit,
            T args,
            System.Windows.Threading.DispatcherPriority priority)
        {
            if (System.Threading.Thread.CurrentThread !=
                ctl.Dispatcher.Thread)
            {
                ctl.Dispatcher.Invoke(priority,
                    doit, args);
            }
            else
            {
                doit(args);
            }
        }

        public static void QueueInvoke(this Control ctl, Action doit)
        {
            ctl.BeginInvoke(doit);
        }

        public static void QueueInvoke<T>(this Control ctl,
            Action<T> doit, T args)
        {
            ctl.BeginInvoke(doit, args);
        }

    }

}

