﻿using System;
using System.Collections.Generic;
using System.Dynamic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Chapter4
{
    class Item44
    {

        public class CSVDataContainer
        {
            private class CSVRow : DynamicObject
            {
                private List<(string, string)> values =
                    new List<(string, string)>();
                public CSVRow(IEnumerable<string> headers,
                    IEnumerable<string> items)
                {
                    values.AddRange(headers.Zip(items,
                        (header, value) => (header,
                             value)));
                }

                public override bool TryGetMember(
                    GetMemberBinder binder,
                    out object result)
                {
                    var answer = values.FirstOrDefault(n =>
                        n.Item1 == binder.Name);
                    result = answer.Item2;
                    return result != null;
                }
            }
            private List<string> columnNames = new List<string>();
            private List<CSVRow> data = new List<CSVRow>();

            public CSVDataContainer(System.IO.TextReader stream)
            {
                // read headers:
                var headers = stream.ReadLine();
                columnNames =
                    (from header in headers.Split(',')
                     select header.Trim()).ToList();

                var line = stream.ReadLine();
                while (line != null)
                {
                    var items = line.Split(',');
                    data.Add(new CSVRow(columnNames, items));
                    line = stream.ReadLine();
                }
            }
            public dynamic this[int index] => data[index];

            public IEnumerable<dynamic> Rows => data; 
        }

        public static void Tests()
        {
            dynamic answer = Add(5, 5);
            Console.WriteLine(answer);
        }

        public static void example()
        {
            int answer = Add(5, 5);
            Console.WriteLine(answer);

            double answer2 = Add(5.5, 7.3);
            Console.WriteLine(answer2);

            // Type arguments needed because
            // args are not the same type
            answer2 = Add<int, double, double>(5, 12.3);
            Console.WriteLine(answer);

            string stringLabel = System.Convert.ToString(answer);

            string label = Add("Here is ", "a label");
            Console.WriteLine(label);

            DateTime tomorrow = Add(DateTime.Now, TimeSpan.FromDays(1));
            Console.WriteLine(tomorrow);

            label = "something" + 3;
            Console.WriteLine(label);
            label = Add("something", 3);
            Console.WriteLine(label);

        }

        public static void ExampleTwo()
        {
            var data = new CSVDataContainer(
                new System.IO.StringReader(myCSV));
            foreach (var item in data.Rows)
                Console.WriteLine(@$"{item.Name}, {item.PhoneNumber}, {item.Label}");

            data = new CSVDataContainer(
                new System.IO.StringReader(myCSV2));
            foreach (var item in data.Rows)
                Console.WriteLine($"{item.Date}, {item.high}, {item.low}");
        }

        public static int Add(int a, int b)
        {
            return a + b;
        }

    private static dynamic DynamicAdd(dynamic left,
        dynamic right) => 
        left + right;

        // Wrap it:
        public static T1 Add<T1, T2>(T1 left, T2 right)
        {
            dynamic result = DynamicAdd(left, right);
            return (T1)result;
        }

        public static TResult Add<T1, T2, TResult>
            (T1 left, T2 right)
        {
            dynamic result = DynamicAdd(left, right);
            return (TResult)result;
        }


    }
}


