﻿using System;
using static System.Console;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Chapter5
{
    class B
    {
        public static B Factory() => new B();

        public virtual void WriteType() => WriteLine("B");
    }

    class D1 : B
    {
        public static new B Factory() => new D1();

        public override void WriteType() => WriteLine("D1");
    }

    class D2 : B
    {
        public static new B Factory() => new D2();

        public override void WriteType() => WriteLine("D2");
    }

    public class Item45
    {
        static void Snippet01()
        {
            string[] labels = new string[] { "one", "two",
                "three", "four", "five" };

            ReplaceIndices(labels);
        }

        static void Snippet02()
        {
            // elsewhere:
            B[]
            storage = new B[10];
            FillArray(storage, () => B.Factory());
            FillArray(storage, () => D1.Factory());
            FillArray(storage, () => D2.Factory());
        }

        static void Snippet03()
        {
            B[] storage = new D1[10];
            // All three calls will throw exceptions:
            FillArray(storage, () => B.Factory());
            FillArray(storage, () => D1.Factory());
            FillArray(storage, () => D2.Factory());
        }

        static private void ReplaceIndices(object[] parms)
        {
            for (int index = 0; index < parms.Length; index++)
                parms[index] = index;
        }

        static private void FillArray(B[] array, Func<B> generator)
        {
            for (int i = 0; i < array.Length; i++)
                array[i] = generator();
        }
    }

    public static class Item45PartTwo
    {
        static void FillArray(D1[] array)
        {
            for (int i = 0; i < array.Length; i++)
                array[i] = new D1();
        }

        public static void Snippet04()
        {
            B[] storage = new B[10];
            // generates compiler error CS1503 (argument mismatch)
            // even though D objects can be placed in a B array
            //FillArray(storage);
        }

        // regular array
        private static void WriteOutput1(object[] stuffToWrite)
        {
            foreach (object o in stuffToWrite)
                Console.WriteLine(o);
        }
        // Params array
        private static void WriteOutput2(params object[] stuffToWrite)
        {
            foreach (object o in stuffToWrite)
                Console.WriteLine(o);
        }

        public static void Snippet05()
        {
            WriteOutput1(new string[]
                { "one", "two", "three", "four", "five" });
            WriteOutput2("one", "two", "three", "four", "five");
        }

        public static void Snippet06()
        {
            WriteOutput2();
            //WriteOutput1(); // won't compile
            WriteOutput1(null); // throws a null argument exception
            WriteOutput1(new object[] { });

        }
    }
}
